﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using STFXPdfDotNET;
using System.IO;
using System.Diagnostics;
using STFXmlParser;

namespace CalcHelper
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            // Somehow this label got 'lost', so I need to do this.
            docLabel.Parent = tableLayoutPanel1;

            // If I collapse this in the designer I do not see the table layout panel
            // on the bottom with it's doc and page labels.
            splitContainer1.Panel2Collapsed = true;
        }

        /*
         *********************************************************
         *
         * Current split screen implementation is as follows:
         * 
         * In single screen mode we use just pdf1 and pa1.
         * 
         * In split screen mode we add pdf2 and pa2 for the second screen panel.
         * We always refer to pdf1/pa1 as the left or top panel, pdf2/pa2 are
         * the right/bottom panel - our screen can be split either horizontally
         * or vertically.
         * 
         * Splitting the screen will automatically display the pdf1 pdf page in the
         * pdf2 panel.
         * 
         * When a document is opened for the pdf1 panel it will automatically be
         * displayed in the pdf2 panel if it is empty [when screen is already split].
         * 
         * You can copy pdf1 over to the pdf2 panel by using the File menu
         * for pdf2. pdf2 can also read it's own pdf, most likely when viewing a
         * document's prior version.
         * 
         * The pdf1 panel has the screen splitting buttons. Clicking on the single
         * screen menu button will collapse the pdf2 panel so you will only see
         * what the pdf1 panel is displaying or blank if it is not occupied.
         * 
         * If you click the single screen menu button and the pdf1 panel is empty and
         * the pdf2's panel is displaying a pdf page, you would see the pdf2's pdf
         * being displayed in the pdf1 panel.
        */
        ProcessPdf pdf1 = null;
        ProcessPdf pdf2 = null;

        PageArray pa1 = null;
        PageArray pa2 = null;

        string pdf1CurDir = "C:\\temp\\Work\\";
        string pdf2CurDir = "C:\\temp\\Work\\";

        bool splitScreen = false;

        /*
         *********************************************************
        */

        /* When the user moves the splitter bar, they cause the 'splitter bar moved' event to
         * fire...once they stop moving the bar. When the user drags the form borders thus resizing
         * the form, a splitter moved event is fired as well as a 'form resize' event. Both
         * events fire off a common event handler. I do not want needless consecutive execution of the
         * common handler. The splitter moving event handler will set this flag and we will look at
         * it in the splitter bar moved handler. If it is set, we hopefully know that the form resizing
         * was not the originator.
        */
        bool splitterMoving = false;

        bool minMaxRestore = false;

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFilePreProcess("pdf1");
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            //MessageBox.Show("Form1_Resize!");

            /*
             * minMaxRestore is only set to true in WndProc() when
             * we detect a Minimize, Maximize, or Restore event.
            */
            if (pdf1 != null)
            {
                if (minMaxRestore)
                {
                    ProcessMinMaxRestore(ref pdf1, ref pa1);
                }
                else
                {
                    ProcessResize(ref pdf1, ref pa1);
                }
            }

            if (pdf2 != null)
            {
                if (minMaxRestore)
                {
                    ProcessMinMaxRestore(ref pdf2, ref pa2);
                }
                else
                {
                    ProcessResize(ref pdf2, ref pa2);
                }
            }

            if (minMaxRestore)
            {
                minMaxRestore = false;
            }
        }

        private void fitInWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessFullPage(ref pdf1, ref pa1);
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Remove previously extracted files and the directory that holds them.
            ProcessCleanupFiles();
        }

        private void openFileBtn_Click(object sender, EventArgs e)
        {
            OpenFilePreProcess("pdf1");
        }

        private void nextPageBtn_Click(object sender, EventArgs e)
        {
            ProcessNextPageNav(ref pdf1, ref pa1);
        }

        private void prevPageBtn_Click(object sender, EventArgs e)
        {
            ProcessPrevPageNav(ref pdf1, ref pa1);
        }

        private void firstPageBtn_Click(object sender, EventArgs e)
        {
            ProcessFirstPageNav(ref pdf1, ref pa1);
        }

        private void lastPageBtn_Click(object sender, EventArgs e)
        {
            ProcessLastPageNav(ref pdf1, ref pa1);
        }

        private void closeFileBtn_Click(object sender, EventArgs e)
        {
            ProcessFileClose(ref pdf1, ref pa1);
        }

        private void fullWidthBtn_Click(object sender, EventArgs e)
        {
            ProcessFullWidth(ref pdf1, ref pa1);
        }

        private void fullPageBtn_Click(object sender, EventArgs e)
        {
            ProcessFullPage(ref pdf1, ref pa1);
        }

        protected override void WndProc(ref Message m)
        {
            if ((pdf1 != null) || (pdf2 != null))
            {
                // WM_SYSCOMMAND?
                if (m.Msg == 0x0112) 
                {
                    int wParam = (m.WParam.ToInt32() & 0xFFF0);
                    bool visible = true;

                    /*
                     * 0xF020 Minimize event - SC_MINIMIZE from Winuser.h
                     * 0xF030 Maximize event - SC_MAXIMIZE from Winuser.h
                     * 0xF120 Restore event - SC_RESTORE from Winuser.h
                    */
                    if ((wParam == 0xF020) || (wParam == 0xF030) || (wParam == 0xF120))
                    {
                        minMaxRestore = true;

                        /*
                         * Check if window WILL be maximized. If so, set
                         * pictureBox1.visible to false so they do not see it
                         * flash on screen before being rescaled and re-displayed
                         * in the Resize event handler. This is the only way I
                         * could find to do this.
                        */
                        if (wParam == 0xF030)
                        {
                            visible = false;
                        }

                        if (pdf1 != null)
                        {
                            pdf1.panel.Visible = visible;
                        }

                        if (pdf2 != null)
                        {
                            pdf2.panel.Visible = visible;
                        }
                    }
                }
            }

            base.WndProc(ref m);
        }

        private void zoomOutBtn_Click(object sender, EventArgs e)
        {
            ProcessZoomOut(ref pdf1, ref pa1);
        }

        private void zoomInBtn_Click(object sender, EventArgs e)
        {
            ProcessZoomIn(ref pdf1, ref pa1);
        }

        private void zoomResetBtn_Click(object sender, EventArgs e)
        {
            ProcessZoomReset(ref pdf1, ref pa1);
        }

        private void pageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessPageToolStripClick(ref pdf1, ref pa1);
        }

        private void boxedPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessBoxedPageToolStripClick(ref pdf1, ref pa1);
        }

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            // Being cautious....
            if (pdf1 == null)
            {
                return;
            }

            //MessageBox.Show(string.Format("MouseDown"));
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ProcessMouseDown(ref pdf1, e.Location);
            }
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            // Being cautious....
            if (pdf1 == null)
            {
                return;
            }

            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ProcessMouseMove(ref pdf1, e.Location);
            }
        }

        private void pictureBox1_MouseClick(object sender, MouseEventArgs e)
        {
            // Being cautious....
            if ((pdf1 == null) || (pa1 == null) || (pdf1.mLassoMove))
            {
                return;
            }

            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ProcessMouseClick(ref pdf1, ref pa1, e.Location);
            }
        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            // Being cautious....
            if ((pdf1 == null) || (pa1 == null))
            {
                return;
            }

            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ProcessMouseUp(ref pdf1, ref pa1);
            }
        }

        public void NavToPageNum(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("NavToPageNum() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("NavToPageNum() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            using (var goToPage = new GoToPage(pdf.numPages))
            {
                var result = goToPage.ShowDialog();

                if (result == DialogResult.OK)
                {
                    pdf.curPage = goToPage.selPage;

                    pdf.ShowPage((int)pdf.curPage, true, ref pa);

                    ProcessPageNum(ref pdf);
                }
            }
        }

        public void NavToBoxedPageNum(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("NavToBoxedPageNum() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("NavToBoxedPageNum() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            using (var goToBoxedPage = new GoToBoxedPage(ref pdf.boxedPageList))
            {
                var result = goToBoxedPage.ShowDialog();

                if (result == DialogResult.OK)
                {
                    pdf.curPage = goToBoxedPage.selBoxedPage;
                    pdf.boxedPageIndex = pdf.boxedPageList.IndexOf(pdf.curPage);

                    pdf.ShowPage((int)pdf.curPage, true, ref pa);

                    ProcessPageNum(ref pdf);
                }
            }
        }

        private void goToPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (pdf1.pageNav)
            {
                NavToPageNum(ref pdf1, ref pa1);
                //MessageBox.Show(string.Format("Back from F5 - Page Nav Mode: {0}", pdf1.curPage));
            }
            else
            {
                NavToBoxedPageNum(ref pdf1, ref pa1);
                //MessageBox.Show(string.Format("Back from F5 - Boxed Page Nav Mode: {0}", pdf1.curPage));
            }
        }

        // Had to do this in order to set the 'View' menus shortcut combinations.
        protected override bool ProcessCmdKey(ref Message msg, Keys keys)
        {
            switch (keys)
            {
                case (Keys.Control | Keys.Add):
                case (Keys.Control | Keys.Oemplus):

                    //MessageBox.Show("Ctrl++");

                    if (zoomInBtn.Enabled)
                    {
                        ProcessZoomIn(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.Add):
                case (Keys.Alt | Keys.Oemplus):

                    //MessageBox.Show("Alt++");

                    if (zoomInBtn2.Enabled)
                    {
                        ProcessZoomIn(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.Subtract):
                case (Keys.Control | Keys.OemMinus):

                    //MessageBox.Show("Ctrl+-");

                    if (zoomOutBtn.Enabled)
                    {
                        ProcessZoomOut(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.Subtract):
                case (Keys.Alt | Keys.OemMinus):

                    //MessageBox.Show("Alt+-");

                    if (zoomOutBtn2.Enabled)
                    {
                        ProcessZoomOut(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.NumPad0):

                    //MessageBox.Show("Ctrl+0");

                    if (zoomResetBtn.Enabled)
                    {
                        ProcessZoomReset(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.NumPad0):

                    //MessageBox.Show("Alt+0");

                    if (zoomResetBtn2.Enabled)
                    {
                        ProcessZoomReset(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.Home):

                    //MessageBox.Show("Ctrl+Home");

                    if (firstPageBtn.Enabled)
                    {
                        ProcessFirstPageNav(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.Home):

                    //MessageBox.Show("Alt+Home");

                    if (firstPageBtn2.Enabled)
                    {
                        ProcessFirstPageNav(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.PageDown):

                    //MessageBox.Show("Ctrl+Page Down");

                    if (nextPageBtn.Enabled)
                    {
                        ProcessNextPageNav(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.PageDown):

                    //MessageBox.Show("Alt+Page Down");

                    if (nextPageBtn2.Enabled)
                    {
                        ProcessNextPageNav(ref pdf2, ref pa2);
                    }
                    break;

                case (Keys.Control | Keys.PageUp):

                    //MessageBox.Show("Ctrl+Page Up");

                    if (prevPageBtn.Enabled)
                    {
                        ProcessPrevPageNav(ref pdf1, ref pa1);
                    }
                    break;

                case (Keys.Alt | Keys.PageUp):

                    //MessageBox.Show("Alt+Page Up");

                    if (prevPageBtn2.Enabled)
                    {
                        ProcessPrevPageNav(ref pdf2, ref pa2);
                    }
                    break;

                default:
                    
                    return base.ProcessCmdKey(ref msg, keys);
            }

            return true;
        }

        private void firstPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessFirstPageNav(ref pdf1, ref pa1);
        }

        private void previousPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessPrevPageNav(ref pdf1, ref pa1);
        }

        private void nextPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessNextPageNav(ref pdf1, ref pa1);
        }

        private void lastPageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessLastPageNav(ref pdf1, ref pa1);
        }

        private void zoomInToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessZoomIn(ref pdf1, ref pa1);
        }

        private void zoomOutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessZoomOut(ref pdf1, ref pa1);
        }

        private void zoomResetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessZoomReset(ref pdf1, ref pa1);
        }

        private void fullWidthToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessFullWidth(ref pdf1, ref pa1);
        }

        private void singleScrnBtn_Click(object sender, EventArgs e)
        {
            ProcessSingleScrnCLick();
        }

        private void splitHorizBtn_Click(object sender, EventArgs e)
        {
            ProcessSplitHorScrnCLick();
        }

        private void splitVertBtn_Click(object sender, EventArgs e)
        {
            ProcessSplitVerScrnCLick();
        }

        private void splitContainer1_SplitterMoving(object sender, SplitterCancelEventArgs e)
        {
            splitterMoving = true;
        }

        private void splitContainer1_SplitterMoved(object sender, SplitterEventArgs e)
        {
            if (splitterMoving)
            {
                /*
                 * This event was triggered when someone stopped moving the
                 * splitter bar as opposed to being triggered due to a form resize
                 * event which ends up calling ProcessResize() too.
                 * 
                 * We need to check both pdf's for null because we can have a split
                 * screen with both panels empty.
                */
                if (pdf1 != null)
                {
                    ProcessResize(ref pdf1, ref pa1);
                }

                if (pdf2 != null)
                {
                    ProcessResize(ref pdf2, ref pa2);
                }

                splitterMoving = false;
            }
        }

        private void openToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            OpenFilePreProcess("pdf2");
        }

        private void copyPaneToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2 != null)
            {
                // Make sure we do not cleanup pdf and xml files if being used by pdf1.
                if (pdf2.ExtPath != pdf1.ExtPath)
                {
                    pdf2.CleanupFiles();
                }

                if (pdf2.zoomCnt > 0)
                {
                    DisableZoomResetBtns(ref pdf2);
                }

                pdf2 = null;
                pa2 = null;
            }

            CopyPdf1ToPdf2(true);
        }

        private void closeToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessFileClose(ref pdf2, ref pa2);
        }

        private void firstPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessFirstPageNav(ref pdf2, ref pa2);
        }

        private void previousPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessPrevPageNav(ref pdf2, ref pa2);
        }

        private void nextPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessNextPageNav(ref pdf2, ref pa2);
        }

        private void lastPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessLastPageNav(ref pdf2, ref pa2);
        }

        private void goToPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            if (pdf2.pageNav)
            {
                NavToPageNum(ref pdf2, ref pa2);
                //MessageBox.Show(string.Format("Back from F5 - Page Nav Mode: {0}", pdf2.curPage));
            }
            else
            {
                NavToBoxedPageNum(ref pdf2, ref pa2);
                //MessageBox.Show(string.Format("Back from F5 - Boxed Page Nav Mode: {0}", pdf2.curPage));
            }
        }

        private void zoomInToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessZoomIn(ref pdf2, ref pa2);
        }

        private void zoomOutToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessZoomOut(ref pdf2, ref pa2);
        }

        private void zoomResetToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessZoomReset(ref pdf2, ref pa2);
        }

        private void fullWidthToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessFullWidth(ref pdf2, ref pa2);
        }

        private void fullPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessFullPage(ref pdf2, ref pa2);
        }

        private void openFileBtn2_Click(object sender, EventArgs e)
        {
            OpenFilePreProcess("pdf2");
        }

        private void closeFileBtn2_Click(object sender, EventArgs e)
        {
            ProcessFileClose(ref pdf2, ref pa2);
        }

        private void firstPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessFirstPageNav(ref pdf2, ref pa2);
        }

        private void prevPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessPrevPageNav(ref pdf2, ref pa2);
        }

        private void nextPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessNextPageNav(ref pdf2, ref pa2);
        }

        private void lastPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessLastPageNav(ref pdf2, ref pa2);
        }

        private void pageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessPageToolStripClick(ref pdf2, ref pa2);
        }

        private void boxedPageToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ProcessBoxedPageToolStripClick(ref pdf2, ref pa2);
        }

        private void zoomInBtn2_Click(object sender, EventArgs e)
        {
            ProcessZoomIn(ref pdf2, ref pa2);
        }

        private void zoomOutBtn2_Click(object sender, EventArgs e)
        {
            ProcessZoomOut(ref pdf2, ref pa2);
        }

        private void zoomResetBtn2_Click(object sender, EventArgs e)
        {
            ProcessZoomReset(ref pdf2, ref pa2);
        }

        private void fullWidthBtn2_Click(object sender, EventArgs e)
        {
            ProcessFullWidth(ref pdf2, ref pa2);
        }

        private void fullPageBtn2_Click(object sender, EventArgs e)
        {
            ProcessFullPage(ref pdf2, ref pa2);
        }

        private void singleScreenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessSingleScrnCLick();
        }

        private void splitScreenHorizontallyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessSplitHorScrnCLick();
        }

        private void splitScreenVerticallyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessSplitVerScrnCLick();
        }

        private void pictureBox2_MouseDown(object sender, MouseEventArgs e)
        {
            // Being cautious....
            if (pdf2 == null)
            {
                return;
            }

            //MessageBox.Show(string.Format("MouseDown"));
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ProcessMouseDown(ref pdf2, e.Location);
            }
        }

        private void pictureBox2_MouseMove(object sender, MouseEventArgs e)
        {
            // Being cautious....
            if (pdf2 == null)
            {
                return;
            }

            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ProcessMouseMove(ref pdf2, e.Location);
            }
        }

        private void pictureBox2_MouseClick(object sender, MouseEventArgs e)
        {
            // Being cautious....
            if ((pdf2 == null) || (pa2 == null) || (pdf2.mLassoMove))
            {
                return;
            }

            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ProcessMouseClick(ref pdf2, ref pa2, e.Location);
            }
        }

        private void pictureBox2_MouseUp(object sender, MouseEventArgs e)
        {
            // Being cautious....
            if ((pdf2 == null) || (pa2 == null))
            {
                return;
            }

            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                ProcessMouseUp(ref pdf2, ref pa2);
            }
        }
    }
}
